searchData={"items":[{"type":"behaviour","title":"ProtoRune.Bot","doc":"The `ProtoRune.Bot` module provides the foundational behavior and macros for building bots\nin the ProtoRune ecosystem. It defines the basic structure for bots and ensures that every bot\nadheres to a consistent interface, with customizable event handling, identifier, and password\nretrieval.\n\nThe bot system integrates with the `ProtoRune.Bot.Server` to manage bot lifecycles, handle\nevents, and manage sessions. Bots can use different strategies for receiving notifications,\nsuch as polling or firehose (currently under development).","ref":"ProtoRune.Bot.html"},{"type":"behaviour","title":"Usage - ProtoRune.Bot","doc":"To create a bot using `ProtoRune.Bot`, you need to define your bot module with the required\ncallbacks: `get_identifier/0`, `get_password/0`, and `handle_event/2`.\n\nHere is an example bot implementation:\n\n```elixir\ndefmodule Walle do\n  use ProtoRune.Bot,\n    name: __MODULE__,\n    strategy: :polling\n\n  require Logger\n\n  @impl true\n  def get_identifier, do: System.get_env(\"IDENTIFIER\")\n\n  @impl true\n  def get_password, do: System.get_env(\"PASSWORD\")\n\n  @impl true\n  def handle_event(event, payload) do\n    Logger.info(\"Event: #{event} with URI: #{inspect(payload[:uri])}\")\n  end\nend\n```\n\nIn this example, `Walle` is a bot that uses the polling strategy to fetch notifications.\nIt retrieves its identifier and password from environment variables and logs any events it receives.","ref":"ProtoRune.Bot.html#module-usage"},{"type":"behaviour","title":"Polling Strategy Events - ProtoRune.Bot","doc":"When using the polling strategy, the bot can receive various types of events triggered by\nnotifications from the Bluesky or ATProto services. Each event type corresponds to a specific\nuser action, and a payload containing relevant data is provided. Below is a list of possible\nevents and their associated payloads:","ref":"ProtoRune.Bot.html#module-polling-strategy-events"},{"type":"behaviour","title":"Event Types and Payloads - ProtoRune.Bot","doc":"- **`:reply`**\n  - Triggered when someone replies to a post involving the bot.\n  - **Payload**:\n    - `:uri` - The URI of the post that was replied to.\n    - `:user` - The user who made the reply.\n    - `:content` - The content of the reply post.\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:1234\", user: \"user123\", content: \"Thanks for your post!\"}\n  ```\n\n- **`:quote`**\n  - Triggered when someone quotes the bot's post.\n  - **Payload**:\n    - `:uri` - The URI of the quoted post.\n    - `:user` - The user who quoted the post.\n    - `:content` - The content of the quote.\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:1234\", user: \"user456\", content: \"Great article!\"}\n  ```\n\n- **`:mention`**\n  - Triggered when the bot is mentioned in a post.\n  - **Payload**:\n    - `:uri` - The URI of the post mentioning the bot.\n    - `:user` - The user who mentioned the bot.\n    - `:content` - The content of the post where the bot was mentioned.\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:5678\", user: \"user789\", content: \"Check out @bot's post!\"}\n  ```\n\n- **`:like`**\n  - Triggered when someone likes a post by the bot.\n  - **Payload**:\n    - `:uri` - The URI of the liked post.\n    - `:user` - The user who liked the post.\n    - `:subject` - The subject of the post that was liked (full post data).\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:1234\", user: \"user123\", subject: %{content: \"Nice post!\"}}\n  ```\n\n- **`:repost`**\n  - Triggered when someone reposts content from the bot.\n  - **Payload**:\n    - `:uri` - The URI of the reposted content.\n    - `:user` - The user who reposted the content.\n    - `:post` - The post that was reposted (full post data).\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:5678\", user: \"user987\", post: %{content: \"Check this out!\"}}\n  ```\n\n- **`:follow`**\n  - Triggered when someone follows the bot.\n  - **Payload**:\n    - `:uri` - The URI of the follow event.\n    - `:user` - The user who followed the bot.\n\n  Example payload:\n  ```elixir\n  %{uri: \"at://did:plc:9876\", user: \"user123\"}\n  ```\n\n- **`:error`**\n  - Triggered when there is an error while processing an event (e.g., failed to fetch a post).\n  - **Payload**:\n    - `:reason` - An atom describing the error.\n\n  Example payload:\n  ```elixir\n  %{reason: {:rate_limited, retry_adter :: integer}}\n  ```","ref":"ProtoRune.Bot.html#module-event-types-and-payloads"},{"type":"behaviour","title":"Callbacks - ProtoRune.Bot","doc":"The following callbacks can be implemented by any bot module that uses `ProtoRune.Bot`:\n\n- `get_identifier/0`: Retrieves the bot's identifier (e.g., username or email). This is used\n  for logging into the service.\n\n- `get_password/0`: Retrieves the bot's password. This is used alongside the identifier\n  to authenticate the bot.\n\n- `handle_event/2`: Handles events that are dispatched to the bot. These events can include\n  mentions, replies, likes, and other interactions that the bot should process.\n\nThe `handle_event/2` function receives:\n- `event`: An atom that represents the type of event (e.g., `:mention`, `:like`, `:reply`).\n- `payload`: A map containing the data related to the event, such as the URI of the post or the user who triggered the event.","ref":"ProtoRune.Bot.html#module-callbacks"},{"type":"behaviour","title":"Optional Callbacks - ProtoRune.Bot","doc":"These callbacks are optional and can be overridden by the bot module:\n\n- `get_identifier/0`: If not implemented, a default error will be raised indicating the callback must be defined.\n- `get_password/0`: Similar to `get_identifier/0`, this must be implemented by the bot if needed for authentication.","ref":"ProtoRune.Bot.html#module-optional-callbacks"},{"type":"behaviour","title":"Bot Lifecycle - ProtoRune.Bot","doc":"The bot is started using `start_link/0`, which initializes the bot server with the provided options.\nThe server handles the bot's session and dispatches messages or events to the bot's defined handlers.\n\nFor instance, starting the bot would look like this:\n\n```elixir\nWalle.start_link()\n```","ref":"ProtoRune.Bot.html#module-bot-lifecycle"},{"type":"behaviour","title":"Customizing the Bot - ProtoRune.Bot","doc":"- **Authentication**: Bots must implement `get_identifier/0` and `get_password/0` to provide authentication details.\n- **Event Handling**: The `handle_event/2` function allows bots to react to different types of events such as mentions, replies, and likes.","ref":"ProtoRune.Bot.html#module-customizing-the-bot"},{"type":"behaviour","title":"Example Workflow - ProtoRune.Bot","doc":"When the bot receives a notification (for example, a new mention), the following happens:\n\n1. The bot's `handle_event/2` callback is called with the event type and payload.\n2. The bot processes the event and can take actions such as replying, liking a post, or logging information.","ref":"ProtoRune.Bot.html#module-example-workflow"},{"type":"behaviour","title":"Notes - ProtoRune.Bot","doc":"- The current implementation supports the polling strategy for fetching notifications. Firehose-based notifications are not yet implemented.\n- Bots should be designed to handle events and messages in a non-blocking manner for efficient performance.","ref":"ProtoRune.Bot.html#module-notes"},{"type":"callback","title":"ProtoRune.Bot.get_identifier/0","doc":"","ref":"ProtoRune.Bot.html#c:get_identifier/0"},{"type":"callback","title":"ProtoRune.Bot.get_password/0","doc":"","ref":"ProtoRune.Bot.html#c:get_password/0"},{"type":"callback","title":"ProtoRune.Bot.handle_event/2","doc":"","ref":"ProtoRune.Bot.html#c:handle_event/2"},{"type":"module","title":"ProtoRune.Bot.Poller","doc":"A GenServer module that handles periodic polling of notifications for a bot, and dispatches\nthese notifications to the appropriate handler functions within the bot.\n\nThe `Poller` connects to the ATProto or Bluesky notification systems and periodically polls\nfor new notifications, processes them, and dispatches them as events to the bot server. It\nhandles various types of notifications including replies, mentions, likes, reposts, and follows.","ref":"ProtoRune.Bot.Poller.html"},{"type":"module","title":"Features - ProtoRune.Bot.Poller","doc":"- Periodic polling of notifications based on a customizable interval.\n- Supports exponential backoff in case of rate limiting or errors.\n- Handles session refresh when required.\n- Dispatches notifications like replies, mentions, quotes, likes, reposts, and follows to the bot.\n- Extensible to handle other types of notifications and custom behavior.","ref":"ProtoRune.Bot.Poller.html#module-features"},{"type":"module","title":"Options - ProtoRune.Bot.Poller","doc":"- `:name` (required) - The name of the GenServer instance.\n- `:interval` (required) - The polling interval in seconds for checking new notifications.\n- `:process_from` - Start polling from a specific date/time.\n- `:last_seen` - The last seen date of notifications.\n- `:cursor` - The cursor to fetch subsequent notifications from the API.\n- `:attempt` - Number of polling attempts, used for backoff.\n- `:server_pid` (required) - The server process that handles events from the poller.\n- `:session` (required) - The session information used to authenticate API requests.","ref":"ProtoRune.Bot.Poller.html#module-options"},{"type":"module","title":"Functions - ProtoRune.Bot.Poller","doc":"- `start_link/1`: Starts the `Poller` process with the given options.\n- `poll_notifications/1`: Fetches the latest notifications from the service and handles them.\n- `handle_notifications/2`: Dispatches each notification to the appropriate event handler in the bot server.\n- `handle_rate_limited/2`: Handles the rate-limiting case by applying exponential backoff before the next poll.\n- `handle_error/2`: Sends error events to the bot server.\n- `dispatch_notification/2`: Dispatches different types of notifications (e.g., replies, quotes, mentions, likes, reposts, follows) to the bot server.","ref":"ProtoRune.Bot.Poller.html#module-functions"},{"type":"module","title":"Example - ProtoRune.Bot.Poller","doc":"You can start the poller like this:\n\n```elixir\nProtoRune.Bot.Poller.start_link([\n  name: :my_bot_poller,\n  interval: 30,\n  session: my_session,\n  server_pid: self()\n])\n```\n\nThe poller will then periodically fetch notifications and dispatch them to the bot server based on the event type.","ref":"ProtoRune.Bot.Poller.html#module-example"},{"type":"module","title":"Backoff Strategy - ProtoRune.Bot.Poller","doc":"The poller implements an exponential backoff strategy when rate-limited or in case of errors.\nThe backoff starts with the defined `interval` and increases exponentially with each failed attempt,\nup to a maximum of 5 minutes.","ref":"ProtoRune.Bot.Poller.html#module-backoff-strategy"},{"type":"module","title":"Internal State - ProtoRune.Bot.Poller","doc":"The `State` struct is used to keep track of:\n- `name`: The name of the poller process.\n- `interval`: The polling interval in seconds.\n- `last_seen`: The last notification timestamp.\n- `cursor`: API cursor for fetching new notifications.\n- `attempt`: The number of failed attempts.\n- `session`: The current session for API requests.\n- `server_pid`: The PID of the server handling the notifications.","ref":"ProtoRune.Bot.Poller.html#module-internal-state"},{"type":"function","title":"ProtoRune.Bot.Poller.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ProtoRune.Bot.Poller.html#child_spec/1"},{"type":"function","title":"ProtoRune.Bot.Poller.start_link/1","doc":"","ref":"ProtoRune.Bot.Poller.html#start_link/1"},{"type":"type","title":"ProtoRune.Bot.Poller.kwargs/0","doc":"","ref":"ProtoRune.Bot.Poller.html#t:kwargs/0"},{"type":"type","title":"ProtoRune.Bot.Poller.option/0","doc":"","ref":"ProtoRune.Bot.Poller.html#t:option/0"},{"type":"module","title":"ProtoRune.Bot.Server","doc":"The `ProtoRune.Bot.Server` module is responsible for managing bot processes in ProtoRune.\nIt handles bot initialization, session management, and event/message dispatching. This\nmodule also integrates with the polling system to retrieve real-time notifications from\nATProto and Bluesky services.\n\nThe bot server can operate in two modes:\n- **Polling**: Periodically fetches notifications using the `ProtoRune.Bot.Poller` module.\n- **Firehose**: (Not yet implemented) Stream real-time events using a websocket-like connection.","ref":"ProtoRune.Bot.Server.html"},{"type":"module","title":"Features - ProtoRune.Bot.Server","doc":"- **Bot Lifecycle Management**: The server manages the entire bot lifecycle, from login\n  and session refresh to handling messages and events.\n- **Polling Strategy**: Supports polling for notifications at regular intervals via the\n  `ProtoRune.Bot.Poller`.\n- **Session Management**: Automatically handles session creation, refresh, and expiration.\n- **Event and Message Handling**: Provides a unified interface for handling events and messages\n  via `handle_message/1` and `handle_event/2`.","ref":"ProtoRune.Bot.Server.html#module-features"},{"type":"module","title":"Options - ProtoRune.Bot.Server","doc":"- `:name` (required) - The name of the bot process.\n- `:lang` - A list of languages the bot supports (default: `[\"en\"]`).\n- `:service` - The service endpoint the bot will connect to (default: `\"https://bsky.social\"`).\n- `:identifier` - The bot's login identifier (e.g., email or username).\n- `:password` - The bot's password for login.\n- `:polling` - Polling configuration (e.g., interval and process_from).\n- `:firehose` - Firehose configuration (not implemented yet).\n- `:strategy` - The bot's strategy for receiving notifications (`:polling` or `:firehose`).","ref":"ProtoRune.Bot.Server.html#module-options"},{"type":"module","title":"Polling Configuration - ProtoRune.Bot.Server","doc":"Polling can be configured with the following options:\n- `:interval` - How often (in seconds) the bot should poll for notifications (default: 5 seconds).\n- `:process_from` - Start processing notifications from a specific timestamp (default: current time).\n\nExample:\n```elixir\nProtoRune.Bot.Server.start_link(\n  name: :my_bot,\n  strategy: :polling,\n  service: \"https://bsky.social\",\n  polling: %{interval: 10}\n)\n```","ref":"ProtoRune.Bot.Server.html#module-polling-configuration"},{"type":"module","title":"Firehose Configuration (Not Implemented) - ProtoRune.Bot.Server","doc":"While not yet available, the firehose strategy will enable real-time notifications using a\nwebsocket connection. Firehose configuration includes:\n- `:relay_uri` - The WebSocket URI for the relay server.\n- `:auto_reconnect` - Automatically reconnect if the connection drops (default: true).\n- `:cursor` - The starting cursor for reading the stream.","ref":"ProtoRune.Bot.Server.html#module-firehose-configuration-not-implemented"},{"type":"module","title":"Functions - ProtoRune.Bot.Server","doc":"- `start_link/1`: Starts the bot process with the given configuration options.\n- `handle_message/2`: Handles incoming messages for the bot.\n- `handle_event/3`: Handles events dispatched to the bot.\n- `format_status/1`: Formats the bot's internal state for debugging.","ref":"ProtoRune.Bot.Server.html#module-functions"},{"type":"module","title":"Session Management - ProtoRune.Bot.Server","doc":"The bot manages its session by authenticating with the ATProto server upon startup.\nIt also refreshes the session token periodically. If the session expires or cannot be\nrefreshed, the bot will stop.","ref":"ProtoRune.Bot.Server.html#module-session-management"},{"type":"module","title":"Example - ProtoRune.Bot.Server","doc":"```elixir\nProtoRune.Bot.Server.start_link([\n  name: :my_bot,\n  strategy: :polling,\n  service: \"https://bsky.social\",\n  identifier: \"my-bot-id\",\n  password: \"super-secret-password\"\n])\n```\n\nThis will start a bot that uses the polling strategy to retrieve notifications from the\nBsky service every 5 seconds.\n\nThe bot can handle messages and events like this:\n\n```elixir\nProtoRune.Bot.Server.handle_message(:my_bot, \"hello\")\nProtoRune.Bot.Server.handle_event(:my_bot, :user_joined, %{user: \"user123\"})\n```","ref":"ProtoRune.Bot.Server.html#module-example"},{"type":"module","title":"Internal State - ProtoRune.Bot.Server","doc":"The server maintains a state that includes:\n- `name`: The bot's name.\n- `service`: The endpoint to connect to.\n- `session`: The session data for making authenticated requests.\n- `poller`: The PID of the polling process (if using the polling strategy).\n- `langs`: The languages the bot supports.","ref":"ProtoRune.Bot.Server.html#module-internal-state"},{"type":"module","title":"Error Handling - ProtoRune.Bot.Server","doc":"- The bot gracefully handles errors such as rate limits and API failures by retrying or\n  stopping the process when necessary.\n- Errors are dispatched as events to the bot, allowing custom error handling.","ref":"ProtoRune.Bot.Server.html#module-error-handling"},{"type":"function","title":"ProtoRune.Bot.Server.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ProtoRune.Bot.Server.html#child_spec/1"},{"type":"function","title":"ProtoRune.Bot.Server.firehose_t/1","doc":"","ref":"ProtoRune.Bot.Server.html#firehose_t/1"},{"type":"function","title":"ProtoRune.Bot.Server.firehose_t!/1","doc":"","ref":"ProtoRune.Bot.Server.html#firehose_t!/1"},{"type":"function","title":"ProtoRune.Bot.Server.get_schema/1","doc":"","ref":"ProtoRune.Bot.Server.html#get_schema/1"},{"type":"function","title":"ProtoRune.Bot.Server.handle_event/3","doc":"","ref":"ProtoRune.Bot.Server.html#handle_event/3"},{"type":"function","title":"ProtoRune.Bot.Server.handle_message/2","doc":"","ref":"ProtoRune.Bot.Server.html#handle_message/2"},{"type":"function","title":"ProtoRune.Bot.Server.options_t/1","doc":"","ref":"ProtoRune.Bot.Server.html#options_t/1"},{"type":"function","title":"ProtoRune.Bot.Server.options_t!/1","doc":"","ref":"ProtoRune.Bot.Server.html#options_t!/1"},{"type":"function","title":"ProtoRune.Bot.Server.polling_t/1","doc":"","ref":"ProtoRune.Bot.Server.html#polling_t/1"},{"type":"function","title":"ProtoRune.Bot.Server.polling_t!/1","doc":"","ref":"ProtoRune.Bot.Server.html#polling_t!/1"},{"type":"function","title":"ProtoRune.Bot.Server.start_link/1","doc":"","ref":"ProtoRune.Bot.Server.html#start_link/1"},{"type":"type","title":"ProtoRune.Bot.Server.firehose_t/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:firehose_t/0"},{"type":"type","title":"ProtoRune.Bot.Server.kwargs/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:kwargs/0"},{"type":"type","title":"ProtoRune.Bot.Server.mapargs/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:mapargs/0"},{"type":"type","title":"ProtoRune.Bot.Server.option/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:option/0"},{"type":"type","title":"ProtoRune.Bot.Server.options_t/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:options_t/0"},{"type":"type","title":"ProtoRune.Bot.Server.polling_t/0","doc":"","ref":"ProtoRune.Bot.Server.html#t:polling_t/0"},{"type":"module","title":"ProtoRune.XRPC.Case","doc":"Yeah, in house string casing","ref":"ProtoRune.XRPC.Case.html"},{"type":"function","title":"ProtoRune.XRPC.Case.camelize/1","doc":"","ref":"ProtoRune.XRPC.Case.html#camelize/1"},{"type":"function","title":"ProtoRune.XRPC.Case.snakelize/1","doc":"","ref":"ProtoRune.XRPC.Case.html#snakelize/1"},{"type":"module","title":"ProtoRune.XRPC.Client","doc":"The `XRPC.Client` module handles executing queries and procedures in the XRPC system. It interacts with external services through HTTP requests and performs response validation and schema parsing. The client supports both GET and POST requests, depending on whether the request is a query or a procedure.","ref":"ProtoRune.XRPC.Client.html"},{"type":"module","title":"Overview - ProtoRune.XRPC.Client","doc":"This module allows:\n- **Executing Queries**: Executes `GET` requests for queries.\n- **Executing Procedures**: Executes `POST` requests for procedures with a body.\n- **Error Handling**: Maps various HTTP response codes to custom error messages.","ref":"ProtoRune.XRPC.Client.html#module-overview"},{"type":"module","title":"Functions - ProtoRune.XRPC.Client","doc":"### `execute/1`\n\nExecutes an XRPC query or procedure.\n\n- For **queries**, it performs a `GET` request and validates the query parameters.\n- For **procedures**, it performs a `POST` request and validates the request body.","ref":"ProtoRune.XRPC.Client.html#module-functions"},{"type":"function","title":"ProtoRune.XRPC.Client.apply_case_map/2","doc":"","ref":"ProtoRune.XRPC.Client.html#apply_case_map/2"},{"type":"function","title":"ProtoRune.XRPC.Client.execute/1","doc":"","ref":"ProtoRune.XRPC.Client.html#execute/1"},{"type":"module","title":"ProtoRune.XRPC.DSL","doc":"The `XRPC.DSL` module provides macros to define queries and procedures for interacting with the XRPC system, simplifying the creation of API methods for querying or performing procedures. It supports building custom XRPC queries and procedures by encoding method names and dynamically generating functions based on user-defined parameters.","ref":"ProtoRune.XRPC.DSL.html"},{"type":"module","title":"Overview - ProtoRune.XRPC.DSL","doc":"The primary purpose of this module is to offer a simple DSL for defining queries and procedures in Elixir projects using XRPC. It automates the generation of functions that initiate XRPC requests, reducing boilerplate code for developers.","ref":"ProtoRune.XRPC.DSL.html#module-overview"},{"type":"module","title":"Key Features: - ProtoRune.XRPC.DSL","doc":"- **Query Definition**: Use the `defquery/1` macro to define an XRPC query method.\n- **Procedure Definition**: Use the `defprocedure/2` macro to define an XRPC procedure method.\n- **Parameter Handling**: The `param/2` macro allows specifying parameters and their types for queries and procedures.\n- **Automatic Method Name Encoding**: Converts method names to function names by snakelizing the last segment of the method.","ref":"ProtoRune.XRPC.DSL.html#module-key-features"},{"type":"module","title":"Macros - ProtoRune.XRPC.DSL","doc":"### `defquery/1`\n\nDefines a query function based on the given method name.\n\n```elixir\ndefquery(\"app.bsky.actor.getProfile\")\n```\n\nThis creates a function that returns a new query for the `app.bsky.actor.getProfile` method.\n\n### `defquery/2`\n\nDefines a query with parameters.\n\n```elixir\ndefquery(\"app.bsky.feed.getFeed\", do: block)\n```\n\nWithin the `block`, you can specify parameters using the `param/2` macro. The generated function will include these parameters in the query.\n\n### `defprocedure/2`\n\nDefines a procedure with parameters.\n\n```elixir\ndefprocedure(\"app.bsky.actor.mute\", do: block)\n```\n\nSimilar to `defquery/2`, you can specify parameters in the `block` using the `param/2` macro. This will generate a function that returns a new procedure with the given method and parameters.\n\n### `param/2`\n\nDefines a parameter for queries or procedures.\n\n```elixir\nparam(:actor_id, :string)\n```\n\nThis specifies a parameter with a key of `actor_id` and a type of `:string`, which will be included in the final query or procedure.","ref":"ProtoRune.XRPC.DSL.html#module-macros"},{"type":"module","title":"Usage Example - ProtoRune.XRPC.DSL","doc":"```elixir\ndefmodule MyApp.Bsky do\n  import XRPC.DSL\n\n  defquery \"app.bsky.actor.getProfile\"\n\n  defprocedure \"app.bsky.actor.mute\" do\n    param :actor_id, :string\n  end\nend\n```\n\nIn this example:\n- `get_profile/0` is generated as a function that creates a query for `app.bsky.actor.getProfile`.\n- `mute/0` is generated as a function that creates a procedure for `app.bsky.actor.mute` with the parameter `:actor_id` of type `:string`.","ref":"ProtoRune.XRPC.DSL.html#module-usage-example"},{"type":"macro","title":"ProtoRune.XRPC.DSL.defprocedure/2","doc":"","ref":"ProtoRune.XRPC.DSL.html#defprocedure/2"},{"type":"macro","title":"ProtoRune.XRPC.DSL.defprocedure/3","doc":"","ref":"ProtoRune.XRPC.DSL.html#defprocedure/3"},{"type":"macro","title":"ProtoRune.XRPC.DSL.defquery/2","doc":"","ref":"ProtoRune.XRPC.DSL.html#defquery/2"},{"type":"macro","title":"ProtoRune.XRPC.DSL.defquery/3","doc":"","ref":"ProtoRune.XRPC.DSL.html#defquery/3"},{"type":"function","title":"ProtoRune.XRPC.DSL.encode_method_name/1","doc":"","ref":"ProtoRune.XRPC.DSL.html#encode_method_name/1"},{"type":"macro","title":"ProtoRune.XRPC.DSL.param/2","doc":"","ref":"ProtoRune.XRPC.DSL.html#param/2"},{"type":"type","title":"ProtoRune.XRPC.DSL.options/0","doc":"","ref":"ProtoRune.XRPC.DSL.html#t:options/0"},{"type":"module","title":"ProtoRune.XRPC.Procedure","doc":"The `XRPC.Procedure` module represents a procedure in the XRPC system, encapsulating the method name, request body, and a parser for validating the body. It provides functions to create and manipulate procedure structures.","ref":"ProtoRune.XRPC.Procedure.html"},{"type":"module","title":"Overview - ProtoRune.XRPC.Procedure","doc":"This module allows for:\n- **Creating Procedures**: Use the `new/2` function to create a new procedure with a method and a parser.\n- **Adding a Body**: Use the `put_body/2` function to attach a validated body to the procedure.\n- **String Representation**: The procedure can be converted to a string that represents its full URL.","ref":"ProtoRune.XRPC.Procedure.html#module-overview"},{"type":"module","title":"Functions - ProtoRune.XRPC.Procedure","doc":"### `new/2`\n\nCreates a new procedure with a given method and parser.\n\n```elixir\nXRPC.Procedure.new(\"app.bsky.actor.mute\", from: MyParser)\n```\n\n### `put_body/2`\n\nAttaches a validated body to the procedure.\n\n```elixir\n{:ok, updated_proc} = XRPC.Procedure.put_body(proc, %{\"actor_id\" => \"123\"})\n```\n\nValidates the body using the specified parser and updates the procedure via [peri](https://hexdocs.pm/peri).","ref":"ProtoRune.XRPC.Procedure.html#module-functions"},{"type":"function","title":"ProtoRune.XRPC.Procedure.new/1","doc":"","ref":"ProtoRune.XRPC.Procedure.html#new/1"},{"type":"function","title":"ProtoRune.XRPC.Procedure.new/2","doc":"","ref":"ProtoRune.XRPC.Procedure.html#new/2"},{"type":"function","title":"ProtoRune.XRPC.Procedure.put_body/2","doc":"","ref":"ProtoRune.XRPC.Procedure.html#put_body/2"},{"type":"function","title":"ProtoRune.XRPC.Procedure.put_header/3","doc":"","ref":"ProtoRune.XRPC.Procedure.html#put_header/3"},{"type":"module","title":"ProtoRune.XRPC.Query","doc":"The `XRPC.Query` module is responsible for defining and managing queries in the XRPC system. It encapsulates the method, parameters, headers, and an optional parser, providing functions to create and manipulate query structures.","ref":"ProtoRune.XRPC.Query.html"},{"type":"module","title":"Overview - ProtoRune.XRPC.Query","doc":"This module allows for:\n- **Creating Queries**: Use `new/1` or `new/2` to create a query with an optional parser.\n- **Adding Parameters and Headers**: Use `put_param/3` and `put_header/3` to add query parameters and headers.\n- **String Representation**: Converts a query to a string URL, including parameters if present.","ref":"ProtoRune.XRPC.Query.html#module-overview"},{"type":"module","title":"Functions - ProtoRune.XRPC.Query","doc":"### `new/1`\n\nCreates a new query with the given method.\n\n```elixir\nXRPC.Query.new(\"app.bsky.actor.getProfile\")\n```\n\n### `new/2`\n\nCreates a new query with a method and a parser for validation.\n\n```elixir\nXRPC.Query.new(\"app.bsky.feed.getFeed\", from: MyParser)\n```\n\n### `put_param/3`\n\nAdds or updates a query parameter.\n\n```elixir\nquery = XRPC.Query.put_param(query, :actor_id, \"123\")\n```\n\n### `put_header/3`\n\nAdds or updates a request header.\n\n```elixir\nquery = XRPC.Query.put_header(query, \"Authorization\", \"Bearer token\")\n```","ref":"ProtoRune.XRPC.Query.html#module-functions"},{"type":"function","title":"ProtoRune.XRPC.Query.add_params/2","doc":"","ref":"ProtoRune.XRPC.Query.html#add_params/2"},{"type":"function","title":"ProtoRune.XRPC.Query.new/1","doc":"","ref":"ProtoRune.XRPC.Query.html#new/1"},{"type":"function","title":"ProtoRune.XRPC.Query.new/2","doc":"","ref":"ProtoRune.XRPC.Query.html#new/2"},{"type":"function","title":"ProtoRune.XRPC.Query.put_header/3","doc":"","ref":"ProtoRune.XRPC.Query.html#put_header/3"},{"type":"function","title":"ProtoRune.XRPC.Query.put_param/3","doc":"","ref":"ProtoRune.XRPC.Query.html#put_param/3"},{"type":"protocol","title":"ProtoRune.XRPC.Schema","doc":"","ref":"ProtoRune.XRPC.Schema.html"},{"type":"function","title":"ProtoRune.XRPC.Schema.parse/2","doc":"","ref":"ProtoRune.XRPC.Schema.html#parse/2"},{"type":"type","title":"ProtoRune.XRPC.Schema.t/0","doc":"All the types that implement this protocol.","ref":"ProtoRune.XRPC.Schema.html#t:t/0"},{"type":"extras","title":"ProtoRune","doc":"# ProtoRune\n\n**ProtoRune** is an Elixir framework and client library for building applications on top of the ATProtocol, including **Bluesky** integration. Whether you're developing **bots**, **labelers**, **moderators**, or custom **app views**, ProtoRune provides a flexible and powerful way to interact with the protocol using Elixir.","ref":"readme.html"},{"type":"extras","title":"Features - ProtoRune","doc":"- **XRPC Client**: Interact with ATProto services through a simple, extensible XRPC client.\n- **Schema Generation**: Automatically generate schemas from ATProto `defs.json` files.\n- **Bots & Labelers**: Create bots for content moderation, automated interactions, and more.\n- **Moderation Tools**: Build labelers and moderation tools integrated with ATProto's labeling system.\n- **Flexible Framework**: Use ProtoRune to build custom ATProto applications, including custom feeds, notifications, and more.","ref":"readme.html#features"},{"type":"extras","title":"Installation - ProtoRune","doc":"Add ProtoRune to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:proto_rune, \"~> 0.1.0\"}\n  ]\nend\n```\n\nRun `mix deps.get` to install it.","ref":"readme.html#installation"},{"type":"extras","title":"Getting Started - ProtoRune","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"1. Setting Up a Simple Query - ProtoRune","doc":"```elixir\ndefmodule MyApp.ProfileFetcher do\n  alias ProtoRune.Session\n  alias ProtoRune.Bsky.Actor.Defs.Profile\n\n  @spec get_profile(Session.t, actor: String.t) :: {:ok, Profile.t} | {:error, term}\n  def get_profile(%Session{} = session, actor: actor) do\n    ProtoRune.Bsky.Actor.get_profile(session, actor: actor)\n  end\nend\n```","ref":"readme.html#1-setting-up-a-simple-query"},{"type":"extras","title":"2. Creating a Bot - ProtoRune","doc":"```elixir\ndefmodule MyBot do\n  use ProtoRune.Bot\n\n  @impl true\n  def handle_message(%{text: \"Hello\"}) do\n    \"Hi there!\"\n  end\nend\n```","ref":"readme.html#2-creating-a-bot"},{"type":"extras","title":"3. Building Moderation Tools - ProtoRune","doc":"```elixir\ndefmodule MyModerator do\n  use ProtoRune.Moderator\n\n  def label_inappropriate_content(content) do\n    # Custom logic to label content\n    ProtoRune.Label.apply_label(content, :inappropriate)\n  end\nend\n```\n\nHere's a section to add a roadmap to the README using checklists:","ref":"readme.html#3-building-moderation-tools"},{"type":"extras","title":"Roadmap - ProtoRune","doc":"- [x] **Basic XRPC Client**\n    - Implement a client to interact with ATProto and Bluesky services via XRPC.\n- [ ] **Schemas Parsing and Definition**\n    - Define schemas like `ProfileBasicView` or even `Session` as structs\n- [ ] **Schema Generation**\n    - Automatically generate Elixir structs and typespecs from `defs.json` schema files.\n- [x] **Authenticated Queries**\n    - Support for authenticated queries using session tokens.\n- [x] **Flexible Query and Procedure Macros**\n    - Macros (`defquery` and `defprocedure`) to simplify the definition of queries and procedures.\n- [ ] **Custom Bots**\n    - Build an easy-to-use interface for creating bots that interact with ATProto services.\n- [ ] **Labeling and Moderation Tools**\n    - Provide built-in support for creating labelers and moderators for content on ATProto.\n- [ ] **Advanced Error Handling**\n    - Improve error handling for query execution and schema validation.\n- [ ] **App Views and Custom Feeds**\n    - Allow developers to create custom feeds and views based on the ATProto ecosystem.\n- [ ] **Documentation and Examples**\n    - Expand library documentation with detailed usage examples for bots, labelers, and moderators.","ref":"readme.html#roadmap"},{"type":"extras","title":"Dynamic Schema Generation - ProtoRune","doc":"ProtoRune includes tools for dynamically generating Elixir modules for ATProto schemas:\n\n1. **Generate Schemas from defs.json**\n    - Use the built-in Mix task to generate Elixir structs and typespecs for the schemas defined in the `defs.json` file.\n    ```bash\n    mix proto_rune.gen.schemas\n    ```\n\n2. **Customizable Structs and Typespecs**\n    - ProtoRune generates user-friendly typespecs for all query and procedure parameters, ensuring type safety and ease of use.","ref":"readme.html#dynamic-schema-generation"},{"type":"extras","title":"Contributing - ProtoRune","doc":"1. Fork the repository.\n2. Create your feature branch (`git checkout -b feature/my-feature`).\n3. Commit your changes (`git commit -am 'Add new feature'`).\n4. Push to the branch (`git push origin feature/my-feature`).\n5. Create a new pull request.","ref":"readme.html#contributing"},{"type":"extras","title":"License - ProtoRune","doc":"ProtoRune is licensed under the MIT License.","ref":"readme.html#license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}